@@ -4,10 +4,11 @@
 #include <r_types.h>
 #include <r_anal.h>
 #include <r_cons.h>
+#include <r_hash.h>
 #include <r_util.h>
 #include <r_reg.h>
+#include <r_egg.h>
 #include <r_bp.h>
-#include <r_db.h>
 #include <r_io.h>
 #include <r_syscall.h>
 
@@ -44,7 +45,8 @@ R_LIB_VERSION_HEADER(r_debug);
 #define PTRACE_SYSCALL PT_STEP
 #endif
 
-
+#define SNAP_PAGE_SIZE 4096
+#define CHECK_POINT_LIMIT 0x100000 //TODO: take the benchmark
 /*
  * states that a process can be in
  */
@@ -106,6 +108,7 @@ typedef enum {
 	R_DEBUG_REASON_SWI,
 	R_DEBUG_REASON_INT,
 	R_DEBUG_REASON_FPU,
+	R_DEBUG_REASON_USERSUSP,
 } RDebugReasonType;
 
 
@@ -117,7 +120,6 @@ typedef struct r_debug_frame_t {
 	ut64 bp;
 } RDebugFrame;
 
-
 typedef struct r_debug_reason_t {
 	int /*RDebugReasonType*/ type;
 	int tid;
@@ -137,6 +139,7 @@ typedef struct r_debug_map_t {
 	char *file;
 	int perm;
 	int user;
+	bool shared;
 } RDebugMap;
 
 typedef struct r_debug_signal_t {
@@ -153,16 +156,67 @@ typedef struct r_debug_desc_t {
 	ut64 off;
 } RDebugDesc;
 
+struct r_debug_snap_diff_t;
+typedef struct r_page_data_t {
+	struct r_debug_snap_diff_t *diff; // Pointing SnapDiff that has this pagedata.
+	ut32 page_off;
+	ut8 *data;
+	ut8 hash[128];
+} RPageData;
+
+struct r_debug_snap_t;
+typedef struct r_debug_snap_diff_t {
+	struct r_debug_snap_t *base;
+	RList *pages; // <RPageData*>
+	RPageData **last_changes; // Last diff entries of each pages
+} RDebugSnapDiff;
+
 typedef struct r_debug_snap_t {
 	ut64 addr;
 	ut64 addr_end;
 	ut8 *data;
 	ut32 size;
+	ut32 page_num;
 	ut64 timestamp;
-	ut32 crc;
+	RHash *hash_ctx;
+	ut8 **hashes; // Hash of each pages
+	RList *history; // <RDebugSnapDiff*>
+	int perm;
 	char *comment;
 } RDebugSnap;
 
+typedef struct r_debug_key {
+	ut64 addr;
+	ut32 id;
+} RDebugKey;
+
+typedef struct r_debug_session_t {
+	RDebugKey key;
+	RListIter *reg[R_REG_TYPE_LAST];
+	RList *memlist; // <RDebugSnapDiff*>
+	/* XXX: DebugSession should have base snapshot of memlist. */
+	//RDebugSnap *base;
+} RDebugSession;
+
+/* Session file format */
+typedef struct r_session_header {
+	ut64 addr;
+	ut32 id;
+	ut32 difflist_len;
+} RSessionHeader;
+
+typedef struct r_diff_entry {
+	ut32 base_idx;
+	ut32 pages_len;
+} RDiffEntry;
+
+typedef struct r_snap_entry {
+	ut64 addr;
+	ut32 size;
+	ut64 timestamp;
+	int perm;
+} RSnapEntry;
+
 typedef struct r_debug_trace_t {
 	RList *traces;
 	int count;
@@ -240,11 +294,13 @@ typedef struct r_debug_t {
 	RList *maps; // <RDebugMap>
 	RList *maps_user; // <RDebugMap>
 	RList *snaps; // <RDebugSnap>
+	RList *sessions; // <RDebugSession>
 	Sdb *sgnls;
 	RCoreBind corebind;
 	// internal use only
 	int _mode;
 	RNum *num;
+	REgg *egg;
 } RDebug;
 
 typedef struct r_debug_desc_plugin_t {
@@ -281,6 +337,8 @@ typedef struct r_debug_info_t {
 typedef struct r_debug_plugin_t {
 	const char *name;
 	const char *license;
+	const char *author;
+	const char *version;
 	//const char **archs; // MUST BE DEPREACTED!!!!
 	ut32 bits;
 	const char *arch;
@@ -303,7 +361,7 @@ typedef struct r_debug_plugin_t {
 	int (*cont)(RDebug *dbg, int pid, int tid, int sig);
 	int (*wait)(RDebug *dbg, int pid);
 	bool (*gcore)(RDebug *dbg, RBuffer *dest);
-	int (*kill)(RDebug *dbg, int pid, int tid, int sig);
+	bool (*kill)(RDebug *dbg, int pid, int tid, int sig);
 	RList* (*kill_list)(RDebug *dbg);
 	int (*contsc)(RDebug *dbg, int pid, int sc);
 	RList* (*frames)(RDebug *dbg, ut64 at);
@@ -362,6 +420,7 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp);
 R_API int r_debug_step(RDebug *dbg, int steps);
 R_API int r_debug_step_over(RDebug *dbg, int steps);
 R_API int r_debug_continue_until(RDebug *dbg, ut64 addr);
+R_API int r_debug_continue_until_nonblock(RDebug *dbg, ut64 addr);
 R_API int r_debug_continue_until_optype(RDebug *dbg, int type, int over);
 R_API int r_debug_continue_until_nontraced(RDebug *dbg);
 R_API int r_debug_continue_syscall(RDebug *dbg, int sc);
@@ -379,7 +438,7 @@ R_API int r_debug_select(RDebug *dbg, int pid, int tid);
 //R_API int r_debug_pid_del(RDebug *dbg);
 //R_API int r_debug_pid_del_thread(RDebug *dbg);
 R_API int r_debug_pid_list(RDebug *dbg, int pid, char fmt);
-R_API RDebugPid *r_debug_pid_new(const char *path, int pid, char status, ut64 pc);
+R_API RDebugPid *r_debug_pid_new(const char *path, int pid, int uid, char status, ut64 pc);
 R_API RDebugPid *r_debug_pid_free(RDebugPid *pid);
 R_API RList *r_debug_pids(RDebug *dbg, int pid);
 
@@ -389,6 +448,8 @@ R_API bool r_debug_use(RDebug *dbg, const char *str);
 R_API RDebugInfo *r_debug_info(RDebug *dbg, const char *arg);
 R_API void r_debug_info_free (RDebugInfo *rdi);
 
+R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file);
+
 /* send signals */
 R_API void r_debug_signal_init(RDebug *dbg);
 R_API int r_debug_signal_send(RDebug *dbg, int num);
@@ -447,7 +508,7 @@ R_API int r_debug_stop(RDebug *dbg);
 /* backtrace */
 R_API RList *r_debug_frames(RDebug *dbg, ut64 at);
 
-R_API int r_debug_is_dead(RDebug *dbg);
+R_API bool r_debug_is_dead(RDebug *dbg);
 R_API int r_debug_map_protect(RDebug *dbg, ut64 addr, int size, int perms);
 /* args XXX: weird food */
 R_API ut64 r_debug_arg_get(RDebug *dbg, int fast, int num);
@@ -489,14 +550,37 @@ R_API int r_debug_esil_watch_empty(RDebug *dbg);
 R_API void r_debug_esil_prestep (RDebug *d, int p);
 
 /* snap */
+R_API RDebugSnap *r_debug_snap_new(void);
 R_API void r_debug_snap_free(void *snap);
 R_API int r_debug_snap_delete(RDebug *dbg, int idx);
 R_API void r_debug_snap_list(RDebug *dbg, int idx, int mode);
-R_API int r_debug_snap_diff(RDebug *dbg, int idx);
 R_API int r_debug_snap(RDebug *dbg, ut64 addr);
-R_API int r_debug_snap_comment (RDebug *dbg, int idx, const char *msg);
+R_API int r_debug_snap_comment(RDebug *dbg, int idx, const char *msg);
+R_API RDebugSnapDiff *r_debug_snap_map(RDebug *dbg, RDebugMap *map);
 R_API int r_debug_snap_all(RDebug *dbg, int perms);
-R_API RDebugSnap* r_debug_snap_get (RDebug *dbg, ut64 addr);
+R_API RDebugSnap *r_debug_snap_get(RDebug *dbg, ut64 addr);
+R_API int r_debug_snap_set_idx(RDebug *dbg, int idx);
+R_API int r_debug_snap_set(RDebug *dbg, RDebugSnap *snap);
+
+/* snap diff */
+R_API void r_debug_diff_free(void *p);
+R_API RDebugSnapDiff *r_debug_diff_add(RDebug *dbg, RDebugSnap *base);
+R_API void r_debug_diff_set(RDebug *dbg, RDebugSnapDiff *diff);
+R_API void r_debug_diff_set_base(RDebug *dbg, RDebugSnap *base);
+
+/* page data */
+R_API void r_page_data_free(void *p);
+
+/* debug session */
+R_API void r_debug_session_free(void *p) ;
+R_API void r_debug_session_list(RDebug *dbg);
+R_API RDebugSession *r_debug_session_add(RDebug *dbg, RListIter **tail);
+R_API void r_debug_session_set(RDebug *dbg, RDebugSession *session);
+R_API void r_debug_session_restore(RDebug *dbg, const char *file);
+R_API bool r_debug_session_set_idx(RDebug *dbg, int idx);
+R_API RDebugSession *r_debug_session_get(RDebug *dbg, RListIter *tail);
+R_API void r_debug_session_save(RDebug *dbg, const char *file);
+R_API int r_debug_step_back(RDebug *dbg);
 
 /* plugin pointers */
 extern RDebugPlugin r_debug_plugin_native;
