@@ -13,6 +13,7 @@
 #define ut8 unsigned char
 #define st8 signed char
 #define boolt int
+
 typedef struct _ut80 {
 	ut64 Low;
 	ut16 High;
@@ -44,6 +45,7 @@ typedef struct _utX{
 #define R_TRUE 1
 #define R_TRUFAE 2
 #define R_NOTNULL (void*)(size_t)1
+#define R_EMPTY { 0 }
 
 /* limits */
 #undef UT64_MAX
@@ -78,9 +80,22 @@ typedef struct _utX{
 #define UT8_MAX  0xFFU
 #define UT8_MIN  0x00U
 
+#define UT64_ALIGN(x) (x + (x - (x % sizeof (ut64))))
+#define UT32_ALIGN(x) (x + (x - (x % sizeof (ut32))))
+#define UT16_ALIGN(x) (x + (x - (x % sizeof (ut16))))
+
 #define UT32_LO(x) ((ut32)((x)&UT32_MAX))
 #define UT32_HI(x) ((ut32)(((ut64)(x))>>32)&UT32_MAX)
 
+/* preventive math overflow checks */
+#if !defined(SZT_ADD_OVFCHK)
+#define SZT_ADD_OVFCHK(x,y) ((SIZE_MAX - (x)) <= (y))
+#endif
+#define UT64_ADD_OVFCHK(x,y) ((UT64_MAX - (x)) <= (y))
+#define UT32_ADD_OVFCHK(x,y) ((UT32_MAX - (x)) <= (y))
+#define UT16_ADD_OVFCHK(x,y) ((UT16_MAX - (x)) <= (y))
+#define UT8_ADD_OVFCHK(x,y) ((UT8_MAX - (x)) <= (y))
+
 /* copied from bithacks.h */
 #define B_IS_SET(x, n)   (((x) & (1<<(n)))?1:0)
 #define B_SET(x, n)      ((x) |= (1<<(n)))
@@ -114,5 +129,11 @@ typedef struct _utX{
 #if !defined(NAN)
 #define NAN (0.0f/0.0f)
 #endif
-
+#ifdef _MSC_VER
+#define R_PACKED( __Declaration__ ) __pragma( pack(push, 1) ) __Declaration__ __pragma( pack(pop) )
+#undef INFINITY
+#undef NAN
+#elif defined(__GNUC__)
+#define R_PACKED( __Declaration__ ) __Declaration__ __attribute__((__packed__))
+#endif
 #endif
